@@ -21,6 +21,7 @@ from .. import klass, osutils
 from ..mappings import ImmutableDict
 from ..obj import popattr
 from ..sequences import split_negations, split_elements
+from ..strings import pluralism as _pl
 from ..version import get_version
 
 
@@ -154,6 +155,11 @@ class CommaSeparatedNegations(argparse._AppendAction):
                 raise argparse.ArgumentTypeError(e)
             disabled.extend(neg)
             enabled.extend(pos)
+        colliding = set(disabled).intersection(enabled)
+        if colliding:
+            collisions = ', '.join(map(repr, sorted(colliding)))
+            msg = f'colliding enabled and disabled value{_pl(colliding)}: {collisions}'
+            raise argparse.ArgumentError(self, msg)
         return disabled, enabled
 
     def __call__(self, parser, namespace, values, option_string=None):
@@ -949,14 +955,12 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
                 (1 for x in range(argv['-v'] + argv['--verbose'])),
             )))
 
-        # subparser to use if none is specified on the command line and one is required
-        self.__default_subparser = None
         # subparsers action object from calling add_subparsers()
         self.__subparsers = None
         # function to execute for this parser
         self.__main_func = None
-        # arg checking function to execute for this parser
-        self.__final_check = None
+        # pre-parse functions to execute for this parser
+        self.__pre_parse = []
 
         # Store parent parsers allowing for separating parsing args meant for
         # the root command with args targeted to subcommands. This enables
@@ -1074,26 +1078,14 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
 
     def _parse_known_args(self, arg_strings, namespace):
         """Add support for using a specified, default subparser."""
-        # parse base command options so selected subparser is the first remaining arg
-        if self.__default_subparser is not None:
+        # run registered pre-parse functions
+        if self.__pre_parse:
+            for functor, parser in self.__pre_parse:
+                functor(parser, namespace)
+            # re-parse to catch any argument additions/changes
             namespace, arg_strings = super().parse_known_optionals(arg_strings, namespace)
-
-        skip_subparser_fallback = (
-            self.__default_subparser is None or  # no default requested
-            {'-h', '--help'}.intersection(arg_strings) or  # help requested
-            (arg_strings and arg_strings[0] in self.subparsers)  # subparser already determined
-        )
-
-        if not skip_subparser_fallback:
-            if self.__default_subparser not in self.subparsers:
-                raise ValueError(
-                    'unknown subparser %r (available subparsers %s)' % (
-                    self.__default_subparser, ', '.join(sorted(self.subparsers))))
-            # parse all options the parent parsers know about
-            for parser in self._parents:
-                namespace, arg_strings = parser._parse_known_args(arg_strings, namespace)
-            # prepend the default subcommand to the current arg list
-            arg_strings = [self.__default_subparser] + arg_strings
+            # wipe pre-parse functions so they only run once
+            del self.__pre_parse[:]
 
         # parse the remaining args
         return super()._parse_known_args(arg_strings, namespace)
@@ -1113,9 +1105,6 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
             # override the function to be run if the subcommand sets one
             if subcmd_parser.__main_func is not None:
                 namespace.main_func = subcmd_parser.__main_func
-            # override the arg checking function if the subcommand sets one
-            if subcmd_parser.__final_check is not None:
-                namespace.final_check = subcmd_parser.__final_check
 
         if unknown_args:
             self.error('unrecognized arguments: %s' % ' '.join(unknown_args))
@@ -1147,10 +1136,11 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
             e = sys.exc_info()[1]
             self.error(str(e))
 
-        final_check = getattr(args, 'final_check', None)
-        if final_check is not None:
-            del args.final_check
-            final_check(self, args)
+        # run final arg validation
+        final_checks = [k for k in args.__dict__.keys() if k.startswith('__final_check__')]
+        for check in final_checks:
+            functor = args.pop(check)
+            functor(self, args)
 
         return args
 
@@ -1196,10 +1186,7 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
             return functor
         return f
 
-    def add_subparsers(self, default=None, **kwargs):
-        # set the default subparser to use
-        self.__default_subparser = default
-
+    def add_subparsers(self, **kwargs):
         # If add_subparsers() has already been called return the previous
         # object as argparse doesn't allow multiple objects of this type.
         if self.__subparsers is not None:
@@ -1213,10 +1200,15 @@ class ArgumentParser(OptionalsParser, CsvActionsParser):
         self.__subparsers = subparsers
         return subparsers
 
+    def bind_pre_parse(self, functor):
+        """Decorator to bind a function for pre-parsing parser manipulation."""
+        self.__pre_parse.append((functor, self))
+        return functor
+
     def bind_final_check(self, functor):
         """Decorator to bind a function for argument validation."""
-        self.set_defaults(final_check=functor)
-        self.__final_check = functor
+        name = f'__final_check__{functor.__name__}'
+        self.set_defaults(**{name: functor})
         return functor
 
 
@@ -1272,3 +1264,19 @@ def existent_dir(value):
         return osutils.abspath(value)
     except EnvironmentError as e:
         raise ValueError(f'while resolving path {value!r}, encountered error: {e}') from e
+
+
+def bounded_int(func, desc, x):
+    """Check if argument is an integer and matches defined bounds."""
+    try:
+        n = int(x)
+    except ValueError:
+        raise argparse.ArgumentTypeError('invalid integer value')
+    if not func(n):
+        raise argparse.ArgumentTypeError(f'must be {desc}')
+    return n
+
+
+def positive_int(x):
+    """Check if argument is a positive integer."""
+    return bounded_int(lambda n: n >= 1, '>= 1', x)
